#! /usr/bin/env python3
# -*- coding: utf-8 -*-

"""
    Algorithmes de tris - Tests avec Pytest
    python3 test_tris.py
"""

import random

import pytest

import tris


__author__ = "Olivier Laurent et Laure Gonnord"
__copyright__ = "ENS de Lyon & Univ Lyon1, 2019"


# liste des fonctions de tri à tester
# ces fonctions doivent trier leur argument en place,
# elles ne retournent rien
TRIS_TESTES = [tris.tri_par_insertion, tris.tri_par_selection,
               tris.tri_rapide1, tris.tri_rapide2,
               tris.tri_fusion1, tris.tri_fusion2]


def is_sorted(tab):
    """teste si tab est trie"""
    for i in range(len(tab) - 1):
        if tab[i] > tab[i + 1]:
            return False
    return True


def is_sorted_of(tab, tabref):
    """teste si tab est la version triée de tabref"""
    test = is_sorted(tab)
    for i in set(tab):
        test = test and tab.count(i) == tabref.count(i)
    return test


def random_tab(size, vmin, vmax):
    """tableau aléatoire de longueur size
       contenant des valeurs entre vmin et vmax"""
    return [random.randint(vmin, vmax) for _ in range(size)]


LG_TESTS = 100  # volume des tests aléatoires effectués
LISTE_TESTS = (# tests avec éléments identiques
               [random_tab(100, 1, 100) for _ in range(LG_TESTS)]
               # tests sans éléments identiques (en général)
             + [random_tab(5, 1, 100) for _ in range(LG_TESTS)]
               # liste triée et liste à l'envers
             + [list(range(300)), [300 - i for i in range(300)]])


@pytest.mark.parametrize('sort_fun', TRIS_TESTES)
@pytest.mark.parametrize('tab_value', LISTE_TESTS)
def test_sorted(sort_fun, tab_value):
    """test si sort_fun(tab_value) a trié tab_value"""
    tab_sort = tab_value.copy()
    sort_fun(tab_sort)
    assert is_sorted_of(tab_sort, tab_value)


#  lancement de pytest
if __name__ == '__main__':
    pytest.main()
